/*
 * CspC - A manipulatable datastructure to hold a constraint satisfaction problem.
 * Part of the RandomCsp program.
 * Copyright (C) 1998 J.I. van Hemert
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * Last revision: 28-Mar-1998
 *
 */


#include "csp.h"



// PUBLIC MEMBERS //



CspC::CspC
(
	NumberOfVariablesT const iNumberOfVariables
)
{
	Create (iNumberOfVariables);

} // CspC::CspC



CspC::CspC
(
	CspC const & iCsp
)
{
	Copy (iCsp);

} // CspC::CspC



CspC const & CspC::operator=
(
	const CspC & iCsp
)
{
	if (this != &iCsp)
	{
		Destroy ();
		Copy (iCsp);
	}
	else
		Error->Give ("trying to copy a problem to itself.", Error->warning);

	return (*this);

} // CspC::operator=



CspC::~CspC ()
{
	Destroy ();

} // CspC::~CspC



void const CspC::Reset ()
{
	unsigned int i, j;

	if (Dirty)
	{
		for (i = 0; i < GetNumberOfVariables (); i++)
			for (j = 0; j < GetNumberOfVariables (); j++)
				if (IsConstraint (i, j))
					RemoveConstraint (i, j);

		for (i = 0; i < GetNumberOfVariables (); i++)
			SetDomainSize (i, 0);

		Dirty = false;
	}

} // CspC::Reset



NumberOfVariablesT const CspC::GetNumberOfVariables () const
{
	return (NumberOfVariables);

} // CspC::GetNumberOfVariables



unsigned int const CspC::GetNumberOfConstraints
(
) const
{
	return (NumberOfConstraints);

} // CspC::GetNumberOfConstraints



DomainSizeT const CspC::GetDomainSize
(
	VariableT const iVariable
) const
{
#ifdef CSPC_DEBUG
	IsValidParameter (iVariable);
#endif

	return Variables [iVariable].DomainSize;

} // CspC::GetDomainSize



void const CspC::SetDomainSize
(
	VariableT const iVariable,
	DomainSizeT const iDomainSize
)
{
#ifdef CSPC_DEBUG
	IsValidParameter (iVariable);
#endif

	Variables [iVariable].DomainSize = iDomainSize;

	Dirty = true;

} // CspC::SetDomainSize



bool const CspC::IsValid
(
	ValueT const iSolution []
) const
{
	unsigned int i, j;

#ifdef CSPC_DEBUG
	if (sizeof (iSolution) < GetNumberOfVariables ())
		Error->Give ("size of provided solution has less variables then the problem hos.", Error->error);

	if (!IsDirty ())
		Error->Give ("testing validity of an empty problem.", Error->warning);
#endif

	for (i = 0; i < GetNumberOfVariables (); i++)
		for (j = 0; j < GetNumberOfVariables (); j++)
			if (IsConflict (i, j, iSolution [i], iSolution [j]))
					return false;

	return true;
} // CspC::IsValid



int const CspC::GetNumberOfConflicts
(
	ValueT const iSolution []
) const
{
	unsigned int i, j;
	unsigned int c = 0;

	for (i = 0; i < GetNumberOfVariables (); i++)
		for (j = i + 1; j < GetNumberOfVariables (); j++)
			if (IsConflict (i, j, iSolution [i], iSolution [j]))
					c++;
	return c;
} // CspC::IsValid



void const CspC::AddConstraint
(
	VariableT const iVariable1,
	VariableT const iVariable2
)
{
	unsigned int i, j = 0;
    VariableT cVariable1 = iVariable1;
    VariableT cVariable2 = iVariable2;

#ifdef CSPC_DEBUG
	IsValidParameter (iVariable1);
	IsValidParameter (iVariable2);
#endif

	if (cVariable1 > cVariable2)
		Swap (cVariable1, cVariable2);

#ifdef CSPC_DEBUG
	if (cVariable1 == cVariable2)
		Error->Give ("trying to add a constraint between a variable and itself.", Error->warning);
	if (Constraints [cVariable1] [cVariable2] != 0)
		Error->Give ("trying to add a second constraint between two variables.", Error->warning);
#endif
	
	NumberOfConstraints++;

	Constraints [cVariable1] [cVariable2] = new ConstraintS;
	Constraints [cVariable1] [cVariable2]->Conflicts  = new ConflictT * [GetDomainSize (cVariable1)];

	for (i = 0; i < GetDomainSize (cVariable1); i++)
	{
		Constraints [cVariable1] [cVariable2]->Conflicts [i] = new ConflictT [GetDomainSize (cVariable2)];

		for (j = 0; j < GetDomainSize (cVariable2); j++)
			Constraints [cVariable1] [cVariable2]->Conflicts [i] [j] = false;
	}

	Dirty = true;

} //CspC::AddConstraint



void const CspC::RemoveConstraint
(
	VariableT const iVariable1,
	VariableT const iVariable2
)
{
	unsigned int i;
    VariableT cVariable1 = iVariable1;
    VariableT cVariable2 = iVariable2;
	
#ifdef CSPC_DEBUG
	if (!IsConstraint (iVariable1, iVariable2))
		Error->Give ("trying to remove a non-existing constraint.", Error->warning);
#endif

	if (cVariable1 > cVariable2)
		Swap (cVariable1, cVariable2);

	if (IsConstraint (cVariable1, cVariable2))
	{
		for (i = 0; i < GetDomainSize (cVariable1); i++)
			delete Constraints [cVariable1] [cVariable2]->Conflicts [i];
		delete Constraints [cVariable1] [cVariable2]->Conflicts;


		NumberOfConstraints--;
	}

	delete Constraints [cVariable1] [cVariable2];
	Constraints [cVariable1] [cVariable2] = 0;

} // CspC::RemoveConstraint



bool const CspC::IsConstraint
(
	VariableT const iVariable1,
	VariableT const iVariable2
) const
{
    VariableT cVariable1 = iVariable1;
    VariableT cVariable2 = iVariable2;
	
#ifdef CSPC_DEBUG
	IsValidParameter (iVariable1);
	IsValidParameter (iVariable2);
#endif

	if (cVariable1 > cVariable2)
		Swap (cVariable1, cVariable2);

	if (Constraints [cVariable1] [cVariable2] == 0)
		return false;
	else
		return true;

} // CspC::IsConstraints



void const CspC::AddConflict
(
	VariableT const iVariable1,
	VariableT const iVariable2,
	ValueT const iValue1,
	ValueT const iValue2
)
{
#ifdef CSPC_DEBUG
	if (!IsConstraint (iVariable1, iVariable2))
			Error->Give ("adding conflict to non-existing constraint.", Error->error);
#endif

	if (iVariable1 <= iVariable2)
	{
		Constraints [iVariable1] [iVariable2]->Conflicts [iValue1] [iValue2] = true;
	}
	else
	{
		Constraints [iVariable2] [iVariable1]->Conflicts [iValue2] [iValue1] = true;
	}

} // CspC::AddConflict



void const CspC::RemoveConflict
(
	VariableT const iVariable1,
	VariableT const iVariable2,
	ValueT const iValue1,
	ValueT const iValue2
)
{
#ifdef CSPC_DEBUG
	if (!IsConstraint (iVariable1, iVariable2))
			Error->Give ("removing conflict from non-existing constraint.", Error->error);
#endif

	if (iVariable1 <= iVariable2)
	{
		Constraints [iVariable1] [iVariable2]->Conflicts [iValue1] [iValue2] = false;
	}
	else
	{
		Constraints [iVariable2] [iVariable1]->Conflicts [iValue2] [iValue1] = false;
	}

} // CspC::RemoveConflict



bool const CspC::IsConflict
(
	VariableT const iVariable1,
	VariableT const iVariable2,
	ValueT const iValue1,
	ValueT const iValue2
) const
{

#ifdef CSPC_DEBUG
	// Not to much checking here, because we want to let the user query
	// conflicts of non-existing constraints.
	IsValidParameter (iVariable1, iValue1);
	IsValidParameter (iVariable2, iValue2);
#endif

	if (IsConstraint (iVariable1, iVariable2))
	{
		if (iVariable1 <= iVariable2)
			return Constraints [iVariable1] [iVariable2]->Conflicts [iValue1] [iValue2];
		else
			return Constraints [iVariable2] [iVariable1]->Conflicts [iValue2] [iValue1];
	}
	else
		return false;
	
} // CspC::IsConflict



bool const CspC::IsDirty () const
{
	return Dirty;

} // CspC::IsDirty



bool const CspC::SaveToFile
(
	char const * iFilename
) const
{
	ofstream OutputFile;
	char * ErrorStr;
	unsigned int i, j, k, l = 0;

#ifdef CSPC_DEBUG
	if (!Dirty)
		Error->Give ("your saving an empty CSP.", Error->warning);
#endif

	OutputFile.open (iFilename, ios::out);

	// Check to see if we have a valid file.
	if (!OutputFile.is_open ())
	{
		ErrorStr = new char [sizeof(iFilename) + 20];
		sprintf(ErrorStr, "could not open %s.", iFilename);
		Error->Give (ErrorStr, Error->ioerror);
		delete ErrorStr;
		return false;
	}

	// Output number of variables and all their domainsizes.
	OutputFile << GetNumberOfVariables () << endl;
	for (i = 0; i < GetNumberOfVariables (); i++)
		OutputFile << GetDomainSize (i) << " ";
	OutputFile << endl << endl;

	// Save all the conflicts.
	for (i = 0; i < GetNumberOfVariables () - 1; i++)
		for (j = i + 1; j < GetNumberOfVariables (); j++)
			for (k = 0; k < GetDomainSize (i); k++)
			{
				for (l = 0; l < GetDomainSize (j); l++)
				{
					if (IsConflict (i, j, k, l))
						OutputFile << "1";
					else
						OutputFile << "0";
				}
				OutputFile << "\n";
			}


	OutputFile.close ();
	
	return true;

} // CspC::SaveToFile



bool const CspC::LoadFromFile
(
	char const * iFilename
)
{
	ifstream InputFile;
	char * ErrorStr;
	unsigned int i, j, k, l = 0;
	char in = '\0';

	InputFile.open (iFilename, ios::in);

	// Check to see if we have a valid file.
	if (!InputFile.is_open ())
	{
		ErrorStr = new char [sizeof(iFilename) + 20];
		sprintf(ErrorStr, "could not open %s.", iFilename);
		Error->Give (ErrorStr, Error->ioerror);
		delete ErrorStr;
		return false;
	}

	// Input number of variables and all their domainsizes.
	Destroy ();
	InputFile >> j;
	Create (j);
	for (i = 0; i < GetNumberOfVariables (); i++)
	{
		InputFile >> j;
	    SetDomainSize (i, j);
	}

	// Kill empty line
	do
		InputFile.get (in);
	while (in != '\n');

	// Load all the conflicts.
	for (i = 0; i < GetNumberOfVariables () - 1; i++)
		for (j = i + 1; j < GetNumberOfVariables (); j++)
			for (k = 0; k < GetDomainSize (i); k++)
				for (l = 0; l < GetDomainSize (j); l++)
				{
					do
					{
						InputFile.get (in);
						if (InputFile.eof ())
						{
							ErrorStr = new char [sizeof(iFilename) + 50];
							sprintf(ErrorStr, "%s ended before the whole problem was read in.", iFilename);
							Error->Give (ErrorStr, Error->ioerror);
							delete ErrorStr;
							return false;
						}
					}
					while (in == '\n');

					if (in == '1')
					{
						if (!IsConstraint (i, j))
							AddConstraint (i, j);
						AddConflict (i, j, k, l);
					}
				}


	InputFile.close ();
	
	return true;

} // CspC::LoadFromFile



// PROTECTED MEMBERS //



CspC::ConstraintT const CspC::GetConstraint
(
	VariableT const iVariable1,
	VariableT const iVariable2
) const
{
	VariableT cVariable1 = iVariable1;
	VariableT cVariable2 = iVariable2;
	
#ifdef CSPC_DEBUG
	if (!IsConstraint (iVariable1, iVariable2))
		Error->Give ("requesting a non-existing constraint, will return a null-pointer.", Error->warning);
#endif

	if (cVariable1 > cVariable2)
		Swap (cVariable1, cVariable2);

	if (IsConstraint (cVariable1, cVariable2))
	{
		return (Constraints [cVariable1] [cVariable2]);
	}
	else
	{
		return (0);
	}

} // CspC::GetConstraint



// PRIVATE MEMBERS //



bool const CspC::IsValidParameter
(
	VariableT const iVariable
) const
{
	if (iVariable > GetNumberOfVariables () - 1)
	{
		Error->Give ("variable found that is not in the set of variables", Error->error);
		return false;
	}

	return true;

} // CspC::IsValidParameter




bool const CspC::IsValidParameter
(
	VariableT const iVariable,
	ValueT const iValue
) const
{
	if (iVariable > GetNumberOfVariables () - 1)
	{
		Error->Give ("variable found that is not in the set of variables", Error->error);
		return false;
	}

	if (iValue > Variables [iVariable].DomainSize - 1)
	{
		Error->Give ("value not in domain of variable found", Error->error);
		return false;
	}

	return true;

} // CspC::IsValidParameter



void const CspC::Create
(
	NumberOfVariablesT const iNumberOfVariables
)
{
	unsigned int i, j = 0;

	Error = new ErrorC ("CspC");

	NumberOfVariables = iNumberOfVariables;
	if (GetNumberOfVariables () < 1)
		Error->Give ("Number of variables should be larger than 0.", Error->error);

	Variables = new VariableS [GetNumberOfVariables ()];
	Constraints = new ConstraintT * [GetNumberOfVariables ()];

	for (i = 0; i < GetNumberOfVariables (); i++)
	{
		SetDomainSize (i, 0);

		Constraints [i] = new ConstraintT [GetNumberOfVariables ()];
		for (j = 0; j < GetNumberOfVariables (); j++)
			Constraints [i] [j] = 0;
	}
	
	NumberOfConstraints = 0;

	Dirty = false;

} // CspC::Create



void const CspC::Destroy ()
{
	unsigned int i, j = 0;

	for (i = 0; i < GetNumberOfVariables (); i++)
		for (j = 0; j < GetNumberOfVariables (); j++)
			if (IsConstraint (i, j))
				RemoveConstraint (i, j);

	for (i = 0; i < GetNumberOfVariables (); i++)
		delete Constraints [i];
	delete Constraints;

	delete Variables;

	delete Error;

} // CspC::Destroy



void const CspC::Copy
(
	CspC const & iCsp
)
{
	unsigned int i, j, k, l = 0;

	Create (iCsp.GetNumberOfVariables ());

	Dirty = iCsp.Dirty;

	for (i = 0; i < GetNumberOfVariables (); i++)
		SetDomainSize (i, iCsp.GetDomainSize (i));

	for (i = 0; i < GetNumberOfVariables () - 1; i++)
		for (j = i + 1; j < GetNumberOfVariables (); j++)
			if (iCsp.IsConstraint (i, j))
			{
				AddConstraint (i, j);
				for (k = 0; k < iCsp.GetDomainSize (i); k++)
					for (l = 0; l < iCsp.GetDomainSize (j); l++)
						if (iCsp.IsConflict (i, j, k, l))
							AddConflict (i, j, k, l);
			}

} // CspC::Copy



void const CspC::Swap
(
	VariableT & iVariable1,
	VariableT & iVariable2
) const
{
	VariableT temp;

	temp = iVariable2;
	iVariable2 = iVariable1;
	iVariable1 = temp;

} // CspC::Swap

// eof csp.cc
